# -*- coding: utf-8 -*-

from mock import patch, Mock
from unittest import TestCase

from alf.managers import TokenManager
from alf.client import Client, BearerTokenAuth


class TestClient(TestCase):

    end_point = 'http://endpoint/token'
    resource_url = 'http://api/some/resource'

    def test_Client_should_have_a_variable_with_a_token_manager_class(self):
        self.assertEquals(Client.token_manager_class, TokenManager)

    def test_token_manager_object_should_be_an_instance_of_token_manager_class(self):
        client = Client(token_endpoint=self.end_point, client_id='client-id', client_secret='client_secret')

        self.assertTrue(isinstance(client._token_manager, client.token_manager_class))

    @patch('alf.client.TokenManager.__init__')
    def test_should_have_token_request_timeout(self, init):
        init.return_value = None

        client = Client(
            token_endpoint=self.end_point,
            client_id='client_id',
            client_secret='client_secret',
            token_request_params={
                'timeout': 10
            }
        )

        init.assert_called_with(client_id='client_id',
                                client_secret='client_secret',
                                token_endpoint=self.end_point,
                                token_request_params={
                                    'timeout': 10
                                },
                                token_retries=None,
                                token_storage=None)

    @patch('alf.client.TokenManager')
    @patch('requests.Session.request')
    def test_should_retry_a_bad_request_once(self, request, Manager):
        request.return_value = Mock(status_code=401)
        self._fake_manager(Manager, has_token=False)

        self._request(Manager)

        self.assertEqual(request.call_count, 2)

    @patch('requests.Session.post')
    @patch('requests.Session.request')
    def test_should_stop_the_request_when_token_fails(self, request, post):
        post.return_value = Mock(status_code=500, ok=False)

        client = Client(
            token_endpoint=self.end_point,
            client_id='client_id',
            client_secret='client_secret'
        )

        response = client.request('GET', self.resource_url)

        self.assertEqual(response.status_code, 500)

    @patch('alf.client.TokenManager.reset_token')
    @patch('requests.Session.post')
    @patch('requests.Session.request')
    def test_should_reset_token_when_token_fails(self, request, post, reset_token):
        post.return_value = Mock(status_code=500, ok=False)

        client = Client(
            token_endpoint=self.end_point,
            client_id='client_id',
            client_secret='client_secret'
        )

        response = client.request('GET', self.resource_url)

        self.assertTrue(reset_token.called)

        self.assertEqual(response.status_code, 500)

    @patch('alf.client.TokenManager._has_token')
    @patch('alf.client.TokenManager.reset_token')
    @patch('requests.Session.request')
    def test_should_reset_token_when_gets_an_unauthorized_error(self, request, reset_token, _has_token):
        request.return_value = Mock(status_code=401)
        _has_token.return_value = True

        client = Client(
            token_endpoint=self.end_point,
            client_id='client_id',
            client_secret='client_secret'
        )

        response = client.request('GET', self.resource_url)

        self.assertTrue(reset_token.called)

        self.assertEqual(response.status_code, 401)

    @patch('requests.Session.request')
    def assertRequestsResource(self, Manager, access_token, status_code, request):
        request.return_value = Mock(status_code=status_code)
        self._request(Manager)

        self.assertTrue(request.called)
        self.assertResourceWasRequested(request.call_args, access_token=access_token)

    def assertResourceWasRequested(self, call, access_token):
        args, kwargs = call
        self.assertEqual(args, ('GET', self.resource_url))

        auth = kwargs['auth']
        self.assertIsInstance(auth, BearerTokenAuth)
        self.assertEqual(auth._access_token, access_token)

    def _request(self, manager):
        class ClientTest(Client):
            token_manager_class = manager

        client = ClientTest(
            token_endpoint=self.end_point,
            client_id='client_id',
            client_secret='client_secret')

        return client.request('GET', self.resource_url)

    def _fake_manager(self, Manager, has_token=True, access_token='', status_code=200):
        if not isinstance(access_token, list):
            access_token = [access_token]

        access_token.reverse()

        manager = Mock()
        manager._has_token.return_value = has_token
        manager.get_token.return_value = access_token[0]
        manager.request_token.return_value = Mock(
            status_code=status_code, ok=(status_code == 200))
        Manager.return_value = manager

        return manager



# -*- coding: utf-8 -*-

from unittest import TestCase

from mock import Mock, patch

from alf.adapters import mount_retry_adapter


class MountRetryAdapterTestCase(TestCase):

    @patch('requests.adapters.HTTPAdapter')
    def test_mounts_HTTP_adapter_with_retry(self, HTTPAdapter):
        number_or_retry_object = Mock()

        adapter = Mock()
        HTTPAdapter.return_value = adapter

        session = Mock()
        mount_retry_adapter(session, retries=number_or_retry_object)

        HTTPAdapter.assert_called_once_with(max_retries=number_or_retry_object)

        session.mount.assert_any_call('http://', adapter)
        session.mount.assert_any_call('https://', adapter)



# -*- coding: utf-8 -*-

import requests


class BearerTokenAuth(requests.auth.AuthBase):

    def __init__(self, access_token):
        self._access_token = access_token

    def __call__(self, request):
        request.headers['Authorization'] = 'Bearer {}'.format(self._access_token)
        return request



# -*- coding: utf-8 -*-

from unittest import TestCase

from freezegun import freeze_time
from mock import Mock, patch

from alf.managers import Token, TokenError, TokenManager


class BaseTokenManagerTestCase(TestCase):
    END_POINT = 'http://endpoint/token'
    CLIENT_ID = 'client_id'
    CLIENT_SECRET = 'client_secret'


class TokenManagerTestCase(BaseTokenManagerTestCase):

    def setUp(self):
        with patch('alf.managers.mount_retry_adapter') as self.mount_adapter:
            self.manager = TokenManager(
                self.END_POINT, self.CLIENT_ID, self.CLIENT_SECRET)

    def test_should_start_with_no_token(self):
        self.assertFalse(self.manager._has_token())

    def test_should_NOT_setup_token_retries_by_default(self):
        self.assertEqual(self.mount_adapter.call_count, 0)

    def test_should_detect_expired_token(self):
        self.manager._token = Token('', expires_on=Token.calc_expires_on(0))
        self.assertFalse(self.manager._has_token())

    def test_should_respect_valid_token(self):
        self.manager._token = Token('', expires_on=Token.calc_expires_on(10))
        self.assertTrue(self.manager._has_token())

    @freeze_time("2015-01-01 10:00:00.117153")
    def test_should_reset_token(self):
        self.manager.reset_token()

        self.assertEqual(self.manager._token.access_token, '')
        self.assertEqual(self.manager._token.expires_on,
                         Token.calc_expires_on(0))

    @patch('requests.Session.post')
    def test_should_be_able_to_request_a_new_token(self, post):
        post.return_value.json.return_value = {
            'access_token': 'accesstoken',
            'expires_on': Token.calc_expires_on(10),
        }

        self.manager._request_token()

        post.assert_called_with(self.END_POINT,
                                timeout=None,
                                data={'grant_type': 'client_credentials'},
                                auth=(self.CLIENT_ID, self.CLIENT_SECRET))

    @patch('requests.Session.post')
    def test_should_have_token_request_timeout(self, post):
        post.return_value.json.return_value = {}
        manager = TokenManager(
            self.END_POINT, self.CLIENT_ID, self.CLIENT_SECRET,
            token_request_params={
                'timeout': 5
            }
        )

        manager._request_token()

        post.assert_called_with(self.END_POINT,
                                timeout=5,
                                data={'grant_type': 'client_credentials'},
                                auth=(self.CLIENT_ID, self.CLIENT_SECRET))

    @patch('requests.Session.post')
    def test_should_raise_token_error_for_bad_token(self, post):
        post.return_value = Mock()
        post.return_value.ok = False
        post.return_value.status_code = 500

        with self.assertRaises(TokenError) as context:
            self.manager._request_token()

        self.assertEqual(context.exception.response.status_code, 500)

    @patch('alf.managers.TokenManager._request_token')
    def test_get_token_data_should_obtain_new_token(self, _request_token):
        self.manager.reset_token()

        self.manager._get_token_data()

        self.assertTrue(_request_token.called)

    @patch('alf.managers.TokenManager._request_token')
    @freeze_time("2015-01-01 10:00:00.117153")
    def test_update_token_should_set_a_token_with_data_retrieved(self, _request_token):
        self.manager.reset_token()
        expires = 100
        _request_token.return_value = {'access_token': 'new_access_token',
                                       'expires_in': expires}

        self.manager._token = Token('access_token',
                                    expires_on=0)

        self.manager._update_token()

        self.assertTrue(_request_token.called)

        self.assertEqual(self.manager._token.access_token, 'new_access_token')
        self.assertEqual(self.manager._token.expires_on, Token.calc_expires_on(expires))

    @patch('alf.managers.TokenManager._request_token')
    def test_update_token_should_set_a_token_with_data_retrieved_from_storage(self, _request_token):
        expires = Token.calc_expires_on(100)
        _request_token.return_value = dict()

        self.manager._token = Token('new_access_token',
                                    expires_on=expires)

        self.manager._token_storage(self.manager._token)

        self.manager._update_token()

        self.assertFalse(_request_token.called)

        self.assertEqual(self.manager._token.access_token, 'new_access_token')
        self.assertEqual(self.manager._token.expires_on, expires)

    def test_should_return_token_value(self):
        self.manager._token = Token('access_token', expires_on=Token.calc_expires_on(10))
        self.assertEqual(self.manager.get_token(), 'access_token')

    @patch('alf.managers.TokenManager._update_token')
    @patch('alf.managers.TokenManager._has_token')
    def test_get_token_should_request_a_new_token_if_do_not_have_a_token(self, _has_token, _update_token):
        _has_token.return_value = False

        self.manager.get_token()

        self.assertTrue(_update_token.called)


class TokenManagerWithRetriesTestCase(BaseTokenManagerTestCase):

    def test_should_start_with_no_token(self):
        with patch('alf.managers.mount_retry_adapter') as self.mount_adapter:
            self.manager = TokenManager(
                self.END_POINT, self.CLIENT_ID, self.CLIENT_SECRET,
                token_retries=44)

        self.mount_adapter.assert_called_once_with(
            self.manager._session, 44)



# -*- coding: utf-8 -*-

import requests

from alf.tokens import Token, TokenError, TokenStorage
from alf.adapters import mount_retry_adapter


class TokenManager(object):

    def __init__(self, token_endpoint, client_id, client_secret,
                 token_storage=None, token_retries=None,
                 token_request_params=None):
        self._token_endpoint = token_endpoint
        self._client_id = client_id
        self._client_secret = client_secret
        self._token_request_params = token_request_params or {}
        self._token_storage = TokenStorage(token_storage)

        self._session = requests.Session()

        if token_retries is not None:
            self._token_retries = token_retries
            mount_retry_adapter(self._session, token_retries)

        self._token = Token()

    def _has_token(self):
        return self._token.is_valid()

    def get_token(self):
        if not self._has_token():
            self._update_token()

        return self._token.access_token

    def _get_token_data(self):
        token_data = self._token_storage.request_token()
        if not token_data:
            token_data = self._request_token()
            expires_in = token_data.get('expires_in', 0)
            token_data['expires_on'] = Token.calc_expires_on(expires_in)
        return token_data

    def reset_token(self):
        self._token = Token()
        self._token_storage(self._token)

    def _update_token(self):
        token_data = self._get_token_data()
        access_token = token_data.get('access_token', '')
        expires_on = token_data.get('expires_on', 0)
        self._token = Token(access_token,
                            expires_on)
        self._token_storage(self._token)

    def _request_token(self):
        response = self._session.post(
            self._token_endpoint,
            data={'grant_type': 'client_credentials'},
            auth=(self._client_id, self._client_secret),
            timeout=self._token_request_params.get('timeout'))

        if not response.ok:
            raise TokenError('Failed to request token', response)

        return response.json()



# -*- coding: utf-8 -*-
import redis
import memcache
import unittest

from alf.managers import Token
from alf.tokens import TokenStorage, TokenDefaultStorage, TOKEN_KEY, TOKEN_EXPIRES
from datetime import datetime, timedelta
from freezegun import freeze_time


class TestToken(unittest.TestCase):

    def setUp(self):
        self.storage_obj = None

    @freeze_time("2015-01-01 10:00:00.117153")
    def test_expires_on_works(self):
        expires = Token.calc_expires_on(10)
        self.assertEqual(datetime.now() + timedelta(seconds=10), expires)

    def test_should_have_an_access_token(self):
        token = Token(access_token='access_token')
        self.assertEqual(token.access_token, 'access_token')

    def test_should_know_when_it_has_expired(self):
        token = Token(access_token='access_token',
                      expires_on=0)
        self.assertFalse(token.is_valid(), self.storage_obj)

    def test_should_know_when_it_is_valid(self):
        token = Token(access_token='access_token',
                      expires_on=Token.calc_expires_on(10))
        self.assertTrue(token.is_valid(), self.storage_obj)


class TestTokenStorage(unittest.TestCase):

    def setUp(self):
        self.storage_obj = TokenDefaultStorage()

    def test_storage_should_add_token(self):
        token = Token(access_token='access_token',
                      expires_on=Token.calc_expires_on(10))
        token_storage = TokenStorage(self.storage_obj)
        token_storage(token)
        self.assertEqual(self.storage_obj.get(TOKEN_KEY), 'access_token')

        expires = str(token.expires_on)
        self.assertEqual(self.storage_obj.get(TOKEN_EXPIRES), expires, self.storage_obj)

    def test_storage_should_add_retrieve_token(self):
        token = Token(access_token='access_token',
                      expires_on=Token.calc_expires_on(10))
        expires = str(token.expires_on)
        token_storage = TokenStorage(self.storage_obj)
        token_storage(token)
        token_requested = token_storage.request_token()
        self.assertEqual(len(token_requested), 2)
        self.assertEqual(token_requested.get('access_token'), 'access_token')
        self.assertEqual(self.storage_obj.get('expires_on'), expires, self.storage_obj)

    def test_storage_should_add_just_once_same_value(self):
        expires_datetime = Token.calc_expires_on(10)
        token = Token(access_token='access_token',
                      expires_on=expires_datetime)
        token2 = Token(access_token='new_access_token',
                       expires_on=expires_datetime)
        expires = str(token.expires_on)
        token_storage = TokenStorage(self.storage_obj)
        token_storage(token)
        token_storage._access_token = 'new_access_token'
        token_storage(token2)
        token_requested = token_storage.request_token()
        self.assertEqual(len(token_requested), 2)
        self.assertEqual(token_requested.get('access_token'), 'access_token')
        self.assertEqual(self.storage_obj.get('expires_on'), expires, self.storage_obj)


class TestTokenRedis(TestTokenStorage):

    def setUp(self):
        self.storage_obj = redis.StrictRedis(host='localhost', port=6379, db=0)
        try:
            self.storage_obj.scan()
        except redis.ConnectionError:
            self.skipTest("You don't have a Redis server")

    def tearDown(self):
        self.storage_obj.delete('access_token')


class TestTokenMemcached(TestTokenStorage):

    def setUp(self):
        self.storage_obj = memcache.Client(['127.0.0.1:11211'], debug=1)
        if not self.storage_obj.get_stats():
            self.skipTest("You don't have a Memcached server")

    def tearDown(self):
        self.storage_obj.delete('access_token')




# -*- coding: utf-8 -*-

from mock import Mock
from unittest import TestCase

from alf.auth import BearerTokenAuth


class TestBearerTokenAuth(TestCase):

    def setUp(self):
        self.request = Mock()
        self.request.headers = {}
        self.auth = BearerTokenAuth('token')

    def test_should_add_authorization_header_to_request(self):
        self.auth(self.request)
        self.assertEqual(self.request.headers['Authorization'], 'Bearer token')

    def test_should_return_the_request_object(self):
        modified_request = self.auth(self.request)
        self.assertEqual(self.request, modified_request)



# -*- coding: utf-8 -*-

import requests


def mount_retry_adapter(session, retries):
    adapter = requests.adapters.HTTPAdapter(max_retries=retries)

    session.mount('http://', adapter)
    session.mount('https://', adapter)



# -*- coding: utf-8 -*-
from datetime import datetime, timedelta

TOKEN_KEY = 'access_token'
TOKEN_VALUE = ''
TOKEN_EXPIRES = 'expires_on'


class TokenError(Exception):

    def __init__(self, message, response):
        super(TokenError, self).__init__(message)
        self.response = response


class Token(object):

    def __init__(self, access_token='', expires_on=0):
        self.access_token = access_token
        self.expires_on = expires_on or self.calc_expires_on()

    def is_valid(self):
        return self.expires_on > datetime.now()

    @staticmethod
    def calc_expires_on(expires_in=0):
        return datetime.now() + timedelta(seconds=int(expires_in))


class TokenStorage(object):
    def __init__(self, custom_storage=None):
        self._storage = custom_storage or TokenDefaultStorage()
        self._access_token = TOKEN_VALUE
        self._expires_on = TOKEN_VALUE

    def __call__(self, token):
        if token.access_token != self._access_token \
                or token.expires_on != self._expires_on:
            self._access_token = token.access_token
            self._expires_on = token.expires_on
            self._storage.set(TOKEN_KEY, token.access_token)
            self._storage.set(TOKEN_EXPIRES, str(token.expires_on))

    def request_token(self):
        self._access_token = self._storage.get(TOKEN_KEY)
        expires_on = self._storage.get(TOKEN_EXPIRES)
        if expires_on:
            self._expires_on = datetime.strptime(expires_on,
                                                 "%Y-%m-%d %H:%M:%S.%f")
        else:
            self._expires_on = datetime.now()
        if self._access_token and self._expires_on > datetime.now():
            return {TOKEN_KEY: self._access_token,
                    TOKEN_EXPIRES: self._expires_on}
        return dict()

class TokenDefaultStorage(object):

    def __init__(self, key=TOKEN_KEY, value=TOKEN_VALUE):
        self.storage = dict()
        self.set(key, value)

    def get(self, key=TOKEN_KEY):
        return self.storage.get(key)

    def set(self, key=TOKEN_KEY, value=TOKEN_VALUE):
        self.storage[key] = value




# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

setup(
    name='alf',
    version='0.7.0',
    description="OAuth Client",
    long_description=open('README.rst').read(),
    keywords='oauth client client_credentials requests',
    author=u'Globo.com',
    author_email='timecore@corp.globo.com',
    url='https://github.com/globocom/alf',
    license='MIT',
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        ],
    packages=find_packages(
        exclude=(
            'tests',
        ),
    ),
    include_package_data=True,
    install_requires=[
        'requests>=1.2.3',
    ],
)
